import {
  Datasource, IEventParam, DateHelper, Metadata, Http,
} from '@zeedhi/core';

import {
  AlertService,
  DialogService,
  Form,
  GridEditable, LoadingService, Modal, ModalService, Select, SelectMultiple,
} from '@zeedhi/common';

import { TekGrid } from '@zeedhi/teknisa-components-common';

export class CustoDeLogisticaController {
  private unidadeCurrentRow: any = {};

  private autoRegisterModal!: Modal;

  public dateVisibility: boolean = false;

  public periodVisibility: boolean = true;

  public dateValue: string | string[] = '';

  public DTVIGENCIA: string = this.unidadeCurrentRow.DTVIGENCIA;

  public filterProd: object = {};

  public ModalUnidadeFilter: object = {};

  public autoModalTitle: string | string[] = '';

  public isDeleting: boolean = true;

  public isAdding: boolean = true;

  public onChangeUnidadeCurrentRow({ component }: IEventParam<Datasource>) {
    this.unidadeCurrentRow = component.currentRow;
  }

  get hasNoUnidade() {
    return !this.unidadeCurrentRow.CDFILIAL;
  }

  get hasUnidade() {
    return !!this.unidadeCurrentRow.CDFILIAL;
  }

  public onOpenFilter({ component }) {
    component.childrenInstances.filter((field) => field.name.includes('filterButton'))[0].click();
    this.dateValue = DateHelper.getValue('TODAY', 'DD-MM-YYYY');
  }

  public onchangeDate({ component }) {
    if (!component.value) {
      this.periodVisibility = false;
      this.dateVisibility = true;
    }
  }

  public onchangePeriod({ component }) {
    if (!component.value || !component.value.length) {
      this.periodVisibility = true;
      this.dateVisibility = false;
    }
  }

  public async setFilterValue() {
    const produtoGrid = Metadata.getInstance<GridEditable>('ProdutoGrid');
    this.filterProd = {
      CDFILIAL: this.unidadeCurrentRow.CDFILIAL ?? null,
      DTVIGENCIA: this.unidadeCurrentRow.DTVIGENCIA ?? null,
      CDFORNECED: this.unidadeCurrentRow.CDFORNECED ?? null,
      DTINICIAL: this.unidadeCurrentRow.DTINICIAL ?? null,
      DTFINAL: this.unidadeCurrentRow.DTFINAL ?? null,
      CDREGIAO: this.unidadeCurrentRow.CDREGIAO ?? null,
      TDATAVIGEN: this.unidadeCurrentRow.TDATAVIGEN ?? null,
    };
    produtoGrid.datasource.setFilter(this.filterProd);
    await produtoGrid.reload();
  }

  public onApplyFilterUnidade({ component }) {
    const filter = component.grid.datasource.filter;
    const gridUnidade = Metadata.getInstance<GridEditable>('UnidadeGrid');
    component.grid.datasource.filter = {
      CDFORNECED: filter.CDFORNECED,
      DTVIGENCIA: filter.DTVIGENCIA ? filter.DTVIGENCIA : null,
      TDATAVIGEN: filter.DTVIGENCIA ? filter.DTVIGENCIA : null,
      CDREGIAO: filter.CDREGIAO ? filter.CDREGIAO : null,
      DTINICIAL: (typeof (filter.PERIODO) !== 'undefined') && filter.PERIODO[0] && (filter.PERIODO.length === 2) ? filter.PERIODO[0] : null,
      DTFINAL: (typeof (filter.PERIODO) !== 'undefined') && filter.PERIODO[1] && (filter.PERIODO.length === 2) ? filter.PERIODO[1] : null,
    };

    gridUnidade.reload();
  }

  public onSaveProducts({ component, event }: IEventParam<TekGrid>) {
    event.preventDefault();
    const rows = component.getEditedRows();
    let error = false;

    rows.forEach((row) => {
      const selectedRow: any = (component as any).getChildInstance('NMPRODUTO_editable_'.concat(row.CDARVPROD)).datasource.currentRowValue;
      row.VRPECUSTOLOG = row.VRPECUSTOLOG ?? 0;
      row.VRVRCUSTOLOG = row.VRVRCUSTOLOG ?? 0;
      row.VRPESEGURLOG = row.VRPESEGURLOG ?? 0;
      row.__is_new = row.CDARVPROD === ' '; // this line needs to stay before the CDARVPROD value
      row.CDARVPROD = row.CDARVPROD !== ' ' ? row.CDARVPROD : selectedRow.CDARVPROD;
      row.CDPRODUTO = row.CDPRODUTO !== ' ' ? row.CDPRODUTO : selectedRow.CDPRODUTO;
      row.DTINIVIGCLOG = row.DTINIVIGCLOG ?? this.dateValue;

      if ((row.VRPECUSTOLOG > 0) && (row.VRVRCUSTOLOG > 0)) {
        error = true;
      }
    });

    if (error) {
      DialogService.show(
        {
          title: 'BLOCK_OPERATION_TITLE',
          type: 'error',
          name: 'ValidationSameVigenceData',
          text: 'WARNING_HAS_PERCENT_AND_VALUE_SAME_VIGENCE',
        },
      );
      return;
    }

    LoadingService.show();

    Http.post((component as any).datasource.route.concat('/save'), rows).then((result) => {
      component.cancelEditedRows();
      LoadingService.hide();
      if (result.data.error) {
        if (result.data.error === 'Impossível adicionar. Registro já existe.') {
          AlertService.show({
            name: 'save-faliure',
            text: 'BLOCK_CANNOT_SAVE_ALREADY_EXISTS',
            color: 'error',
            timeout: 5000,
            showDismiss: true,
            top: true,
            right: true,
          });
        }
      } else {
        component.reload();
        AlertService.show({
          name: 'save-success',
          text: 'SUCCESS_SAVE',
          color: 'success',
          timeout: 5000,
          showDismiss: true,
          top: true,
          right: true,
        });
      }
    });
  }

  public addNewProductsRow({ component }: IEventParam<TekGrid>) {
    const gridUnidade = Metadata.getInstance<GridEditable>('UnidadeGrid');
    component.addNewRow({
      CDARVPROD: ' ',
      CDFILIAL: gridUnidade.datasource.currentRow.CDFILIAL,
      CDFORNECED: gridUnidade.datasource.currentRow.CDFORNECED,
      CDPRODUTO: ' ',
      VRPECUSTOLOG: 0,
      VRVRCUSTOLOG: 0,
      VRPESEGURLOG: 0,
    }, 'start');
  }

  public saveProductsInMass(form: Form) {
    DialogService.hide();
    const produtoGrid = Metadata.getInstance<TekGrid>('ProdutoGrid');
    const cdProdField = Metadata.getInstance<Select>('CDARVPROD');
    form.value.VRPECUSTOLOG = form.value.VRPECUSTOLOG ?? 0;
    form.value.VRVRCUSTOLOG = form.value.VRVRCUSTOLOG ?? 0;
    form.value.VRPESEGURLOG = form.value.VRPESEGURLOG ?? 0;
    form.value.CDPRODUTO = (cdProdField as any).selectValue.CDPRODUTO;

    try {
      LoadingService.show();
      Http.post('/saveInMassCustoLogistico', form.value).then(() => {
        this.closeAutoRegisterModal();
        produtoGrid.reload();
        LoadingService.hide();
        AlertService.show({
          name: 'submit-alert',
          text: 'SUCCESS_SAVE',
          color: 'success',
          timeout: 5000,
          showDismiss: true,
          top: true,
          right: true,
        });
      });
    } catch (error) {
      LoadingService.hide();
      AlertService.show({
        name: 'error-alert',
        text: error,
        color: 'error',
        timeout: 5000,
        showDismiss: true,
        top: true,
        right: true,
      });
    }
  }

  public showDialogBeforeSave() {
    const form = Metadata.getInstance<Form>('autoRegisterForm');
    const { VRPECUSTOLOG, VRVRCUSTOLOG } = form.value;
    if ((VRPECUSTOLOG > 0) && (VRVRCUSTOLOG > 0)) {
      DialogService.show(
        {
          title: 'BLOCK_OPERATION_TITLE',
          type: 'error',
          name: 'ValidationSameVigenceData',
          text: 'WARNING_HAS_PERCENT_AND_VALUE_SAME_VIGENCE',
        },
      );
      return;
    }
    if (!form.validate()) return;

    DialogService.show({
      name: 'save-in-mass-confirmation',
      type: 'warning',
      title: 'CONFIRM_OPERATION_TITLE',
      text: 'CONFIRM_SAVE',
      buttons: [
        {
          name: 'yes',
          component: 'ZdButton',
          label: 'YES',
          events: {
            click: () => this.saveProductsInMass(form),
          },
          outline: true,
        },
        {
          name: 'no',
          component: 'ZdButton',
          label: 'NO',
          events: {
            click: () => DialogService.hide(),
          },
        },
      ],
    });
  }

  public deleteProductsInMass() {
    try {
      const produtoGrid = Metadata.getInstance<TekGrid>('ProdutoGrid');
      const form = Metadata.getInstance<Form>('autoRegisterForm');
      const cdProdField = Metadata.getInstance<Select>('CDARVPROD');
      form.value.CDPRODUTO = (cdProdField as any).selectValue.CDPRODUTO;
      LoadingService.show();
      Http.post('/deleteInMassCustoLogistico', form.value).then(() => {
        this.closeAutoRegisterModal();
        produtoGrid.reload();
        LoadingService.hide();
        AlertService.show({
          name: 'submit-alert',
          text: 'SUCCESS_DELETE',
          color: 'success',
          timeout: 5000,
          showDismiss: true,
          top: true,
          right: true,
        });
      });
    } catch (error) {
      LoadingService.hide();
      AlertService.show({
        name: 'error-alert',
        text: error,
        color: 'error',
        timeout: 5000,
        showDismiss: true,
        top: true,
        right: true,
      });
    }
  }

  public deleteProducts({ row }: IEventParam<TekGrid>) {
    const produtoGrid = Metadata.getInstance<any>('ProdutoGrid');
    LoadingService.show();
    Http.delete(produtoGrid.datasource.route.concat('/delete'), { data: row }).then(() => {
      produtoGrid.reload();
      LoadingService.hide();
      AlertService.show({
        name: 'delete-alert',
        text: 'SUCCESS_DELETE',
        color: 'success',
        timeout: 5000,
        showDismiss: true,
        top: true,
        right: true,
      });
    });
  }

  public async createAutoRegisterModal() {
    this.autoRegisterModal = await ModalService.createFromJson('components/custoDeLogisticaAutomaticOperationsModal', true);
  }

  public destroyAutoRegisterModal() {
    this.autoRegisterModal?.destroy();
  }

  public openAutoRegisterModal({ component }) {
    this.isDeleting = component.name === 'autoDeleteProducts';
    this.isAdding = !this.isDeleting;
    this.autoModalTitle = component.name === 'autoAddProducts' ? 'AUTO_REGISTER_TITLE' : 'AUTO_DELETE_TITLE';
    this.autoRegisterModal.show();
  }

  public closeAutoRegisterModal() {
    this.autoRegisterModal.hide();
  }

  public async setFilterUnidadeModal({ component }: any) {
    const unidadeField = Metadata.getInstance<SelectMultiple>('CDFILIAL');
    const arrayRegionToFormat = component.selectedValue.map((value) => value.CDREGIAO);

    this.ModalUnidadeFilter = {
      CDREGIAO: [
        {
          operation: 'IN',
          value: arrayRegionToFormat,
        },
      ],
    };
    await unidadeField.datasource.get();
  }

  public formatMutiple(array: any[]): string {
    return array.length !== 0 ? String('_').concat(array.join('_').concat(String('_'))) : '';
  }
}
